{-----------------------------------------------------------------------

  Advanced Splitter

  Version 1.0, Nov 5, 1997
   - initial release
   - resizes the controls visually when the splitter is moved
   - does not have problems with invisible controls (like the Delphi 3 splitter)

-----------------------------------------------------------------------}
unit AdvSplitter;

interface

uses  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms;

{$R AdvSplitter.res}
{ $ObjExportAll On}

type

  TAdvSplitter = class(TGraphicControl)
  private
    FMouseDown: Boolean;
    FDownPos  : TPoint;
    FControl  : TControl;
    FOnMoved  : TNotifyEvent;
    FMinSize  : Integer;
    FBeveled  : Boolean;
  protected
    procedure Paint; override;
    procedure SetBeveled(Value: Boolean);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align default alLeft;
    property Beveled: Boolean read FBeveled write SetBeveled default True;
    property Color;
    property MinSize: Integer read FMinSize write FMinSize default 30;
    property ParentColor;
    property OnMoved: TNotifyEvent read FOnMoved write FOnMoved;
  end;

procedure Register;

implementation

constructor TAdvSplitter.Create( AOwner : TComponent );
begin
  inherited Create(AOwner);
  FMouseDown := False;
  Align := alLeft;
  Width := 4;
  FControl := nil;
  Cursor := crHSplit;
  FMinSize := 30;
  FBeveled := False;
end;

procedure TAdvSplitter.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

  function FindControl: TControl;
  var
    P: TPoint;
    I: Integer;
  begin
    Result := nil;
    P := Point(Left, Top);
    case Align of
      alLeft: Dec(P.X);
      alRight: Inc(P.X, Width);
      alTop: Dec(P.Y);
      alBottom: Inc(P.Y, Height);
    else
      Exit;
    end;
    for I := 0 to Parent.ControlCount - 1 do
    begin
      Result := Parent.Controls[I];
      if PtInRect(Result.BoundsRect, P) and (Result.Visible) then Exit;
    end;
    Result := nil;
  end;

begin
  inherited;
  if Button = mbLeft then begin
    FMouseDown := True;
    FControl := FindControl;
    FDownPos := Point(X, Y);
  end;
end;

procedure TAdvSplitter.MouseMove(Shift: TShiftState; X, Y: Integer);

  function Max(A,b: Integer): Integer;
  Begin Result:=A; If B>A then Result:=B End;

begin
  inherited;
  if (not FMouseDown) or (not Assigned(FControl)) then exit;
  case Align of
    alLeft  : FControl.Width  := Max(FControl.ScreenToClient(ClientToScreen(Point(X,Y))).X, MinSize);
    alRight : FControl.Left   := Max(Fcontrol.Parent.ClientWidth-FControl.Parent.ScreenToClient(ClientToScreen(Point(X,Y))).X-Width, MinSize);
    alTop   : FControl.Height := Max(FControl.ScreenToClient(ClientToScreen(Point(X,Y))).Y, MinSize);
    alBottom: FControl.Height := Max(Fcontrol.Parent.ClientHeight-FControl.Parent.ScreenToClient(ClientToScreen(Point(X,Y))).Y-Height, MinSize);
  end;//case
end;

procedure TAdvSplitter.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
  if FMouseDown then begin
    FMouseDown := False;
    if Assigned(FOnMoved) then
      FOnMoved(Self);
  end;
end;

procedure TAdvSplitter.Paint;
var
  FrameBrush: HBRUSH;
  R: TRect;
begin
  R := ClientRect;
  Canvas.Brush.Color := Color;
  Canvas.FillRect(ClientRect);
  if Beveled then
  begin
    if Align in [alLeft, alRight] then
      InflateRect(R, -1, 2) else
      InflateRect(R, 2, -1);
    OffsetRect(R, 1, 1);
    FrameBrush := CreateSolidBrush(ColorToRGB(clBtnHighlight));
    FrameRect(Canvas.Handle, R, FrameBrush);
    DeleteObject(FrameBrush);
    OffsetRect(R, -2, -2);
    FrameBrush := CreateSolidBrush(ColorToRGB(clBtnShadow));
    FrameRect(Canvas.Handle, R, FrameBrush);
    DeleteObject(FrameBrush);
  end;
end;

procedure TAdvSplitter.SetBeveled(Value: Boolean);
begin
  FBeveled := Value;
  Repaint;
end;

procedure Register;
begin
   RegisterComponents('Additional', [TAdvSplitter]);
end;

end.
